// This is the main DLL file.

#include "stdafx.h"

#include "ContrastPlugIn.h"
using namespace Drawing;
using namespace ContrastPlugIn;
using namespace Imaging;

// This is the main DLL file. 
#include "stdafx.h"
#include "ContrastPlugIn.h"
using namespace Drawing;
using namespace ContrastPlugIn;

/// <summary>
/// Method that does the Contrast calculation
/// based on the provided parameters.
/// This method is invoked by the application.
/// </summary>
/// <requirements>F:image_contrast_and_color</requirements>
// <param name="param">Parameter which corresponds to the trackBar position.</param>
// <param name="unused1">Not used.</param>
// <param name="unused2">Not used.</param>
void Contrast::ProcessImage(Bitmap& image, 
              int param, int unused1, int unused2)
{
  // Image dimensions.
  unsigned int maxX = image.Width;
  unsigned int maxY = image.Height;
  // Rectangle the size of the image
  Rectangle imageRect;
  imageRect.X = 0;
  imageRect.Y = 0;
  imageRect.Width = maxX;
  imageRect.Height = maxY;
  Rectangle rect(0, 0, maxX, maxY);
  // 8 bit pointer to image data
  unsigned char* imagePtr;
  
  //image data
  BitmapData* imageData;
  
  // Lock the image data before asking for a pointer;
  imageData = image.LockBits(
    rect,
    ImageLockMode::ReadWrite,
    PixelFormat::Format24bppRgb);
  // Scan0.ToPointer returns a pointer to the image data 
  // as void* which is cast to 8 bpp or unsigned char *.
  imagePtr = (unsigned char*)imageData->Scan0.ToPointer();

  // Pixel count initialized with 0.
  unsigned long totNumPixels = 0;
  // Number of possible color values.
  unsigned int maxPixValue = 0;
  // Stores color value of Pixel.
  Color mycolor;
  // stores the new color values.
  int red, blue, green = 0;
  // Slider Value normalized.
  const double step = ((double)param)/100.0;
  // Correction factor.
  double factor = (1.0 + step);

  int sum = 0;
  int pixVal = 0;
  double lutVal = 0;
  
  // Constant values for luminance calculation.
  const double rFactor = 0.299;
  const double gFactor = 0.587;
  const double bFactor = 0.114;
  // Just formats which have 
  // R, G and B values stored as 
  // 8 bits per pixel are supported.
  switch(image.PixelFormat)
  {
  case(Imaging::PixelFormat::Format24bppRgb):
  case(Imaging::PixelFormat::Format32bppArgb):
  case(Imaging::PixelFormat::Format32bppRgb):
  case(Imaging::PixelFormat::Format32bppPArgb):
    maxPixValue = 255;
    break;
  default:
    throw(new Exception("Error, Pixel format not supported for this operation!"));
    break;
  }
  // Define the histogram and the look up table
  unsigned int* histogram = new unsigned int[maxPixValue];
  unsigned int* lut = new unsigned int[maxPixValue];
  
  // Initialize all values in the
  // histogram to 0.
  for( unsigned int q = 0; q <= maxPixValue; ++q)
  {
    histogram[q] = 0;
  }

    // Walk through the whole image
  // and calculate histogram.
  for(unsigned int i = 0; i < maxY; ++i)
  {
    for(unsigned int j = 0; j < maxX*3; j += 3)
    {
      // Calculate the luminance value
      red = imagePtr[imageData->Stride * i + j];
      green = imagePtr[imageData->Stride * i + j + 1];
      blue = imagePtr[imageData->Stride * i + j + 2];
      // Calculate luminance value of current pixel
      pixVal = (int)(((double)(red) * rFactor) + 
        ((double)(green) * gFactor) + 
        ((double)(blue) * bFactor));
      // Add the pixel to the histogram
      histogram[pixVal] += 1;
      // Count number of pixels in
      // the histogram.
      totNumPixels++;
    }
  }


 // Walk through all the values in the histogram
  for( unsigned int k = 0; k <= maxPixValue; ++k)
  {
    for( unsigned int l = 0; l < k; ++l)
    {
      // Add all pixels that were
      // accounted for already
      sum += histogram[l];
    }
    // Calculate the look up table.
    // Accumulate all pixls with a 
    // lower value * maximum value
    // devided by the total number of pixels in the
    // image result in the new pixel value of the current
    // gray value.
    lut[k] = (int)(((double)sum*(double)maxPixValue)/
                      (double)totNumPixels);
    // reset the sum.
    sum = 0;
  }
// Walk through the whole image
  // and calculate the new value of each pixel.
  for(unsigned int i = 0; i < maxY; ++i)
  {
    // Three values for each pixel (RGB).
    for(unsigned int j = 0; j < maxX*3; j += 3)
    {
      red = imagePtr[imageData->Stride * i + j];
      green = imagePtr[imageData->Stride * i + j + 1 ];
      blue = imagePtr[imageData->Stride * i + j + 2];
      // Calculate Luminance value of current Pixel.
      pixVal = (int)(((double)red * rFactor) + 
        ((double)green * gFactor) + 
        ((double)blue * bFactor));
      // Get the lookup table value for the current pixel.
      lutVal = (double)lut[pixVal];
      
      // Calculate the new red, green, and blue values.
      imagePtr[imageData->Stride * i + j ] = 
        calcNewValue(red, rFactor, pixVal, factor, 
        maxPixValue, lutVal);
      imagePtr[imageData->Stride * i + j + 1] = 
        calcNewValue(green, gFactor, pixVal, factor, 
        maxPixValue, lutVal);
      imagePtr[imageData->Stride * i + j + 2] = 
        calcNewValue(blue, bFactor, pixVal, factor, maxPixValue, 
        lutVal);
    }
  }
  // Unlock the image in order
  // to copy the new data into the bitmap.
  image.UnlockBits(imageData);
}

unsigned int 
Contrast::calcNewValue(unsigned int colorValue, 
double colorFactor, unsigned int pixValue, 
double correctionFactor, unsigned int maxPixValue, 
                double lookUpValue)
  {
    // Calculate the new color value for the current pixel.
    // Take the source pixel color
    // distribution into account. 
    if(pixValue < 1)
      pixValue = 1;
    double result = (((double)colorValue*lookUpValue)/
            (double)pixValue) *correctionFactor;
    // Check for overflow
    result = ((result < maxPixValue) ? result : maxPixValue);
    // Check for underflow
    result = ((result < 0) ? 0 : result);
    
    return (unsigned int)result;
  }
