using System;
using System.Drawing;
using System.Windows.Forms;
using System.Threading;
using System.Collections;

namespace Photo_Editor_Application
{
    /// <summary>
    /// Picture class stores all image related data.
    /// It also provides the basic functionality.
    /// </summary>
    /// <requirement>F:image_format</requirement>
	public class Picture : GraphicsComponent
	{
        // private fields used internally
        private Bitmap loadedImage;
        private OpenFileDialog loadFileDialog;
        private SaveFileDialog saveFileDialog;

        /// <summary>
        /// Picture Class contructor.
        /// The image class stores the image and is
        /// responsible for basic image operations.
        /// </summary>
		public Picture()
		{
            //
            // TODO: Add constructor logic here
            //
            string defaultImage = PhotoEditorForm.GetApplicationDirectory + @"\Hawaii.jpg";
            // Load defaul image and throw exception if not found
            try
            {
                
                LoadImage(defaultImage);
            }
            catch(Exception e)
            {
                Exception b = e;
                throw(new Exception("Check if default image Hawaii.jpg is in the path!"));
            }

           if(loadedImage == null)
           {
                throw(new Exception("Error, Image memory allocation failed"));
           }
            // Create an instance of a file dialog for the load image, throw exception if fails.
            loadFileDialog = new OpenFileDialog();
            if(loadFileDialog == null)
            {
                throw(new Exception("Error, OpenFileDialog allocation failed"));
            }

            // Create an intsance of a SaveDialog, exception if fails
            saveFileDialog = new SaveFileDialog();
            if(saveFileDialog == null)
            {
                throw(new Exception("Error, SaveFileDialog allocation failed"));
            }
            // List of components initially set to 20.
            compList = new ArrayList(20);
        }

        /// <summary>
        /// Opens Open File Dialog and loads the selected image.
        /// </summary>
        /// <requirement>F:image_load_and_save</requirement>
        public void LoadImage()
        {	
            // Define file types shown in load dialog.
            loadFileDialog.Filter = " jpg files (*.jpg)|*.jpg| gif files (*.gif)|*.gif| bmp files (*.bmp)|*.bmp| All files (*.*)|*.*";
            // Make the dialog show up on the screen.
            loadFileDialog.ShowDialog();
            // Load the image in the selected file and store it.
            loadedImage = new Bitmap(loadFileDialog.FileName);
            // Check if memory allocation succeeded.
            if(loadedImage == null)
            {
                throw(new Exception("Error, LoadImage failed"));
            }
		}

        /// <summary>
        /// Method called from CropDialog. The current
        /// shown image is cropped to the size provided
        /// in the parameters. The cropping is done
        /// with a rectangle which center is put on
        /// the center of the image.
        /// </summary>
        /// <requirement>F:image_crop</requirement>
        /// <param name="newHeight">height of the cropped image</param>
        /// <param name="newWidth">width of the cropped image</param>
        public void CropImage(int newWidth, int newHeight)
        {
            // Check that corpping region is
            // actually within the image and that
            // the values provided are positive
            if((newWidth < loadedImage.Size.Width) && 
                (newHeight < loadedImage.Size.Height) &&
                (newHeight > 0) && (newWidth > 0))
            {
                int xmin = (loadedImage.Size.Width / 2) - 
                    (newWidth / 2);
                int xdim = newWidth;
                int ymin = (loadedImage.Size.Height / 2) - 
                    (newHeight / 2);
                int ydim =	newHeight;
                // Defines the rectangle which is the cropping area.
                Rectangle rectangle = new Rectangle(xmin, ymin, 
                    xdim, ydim);
                if(rectangle.IsEmpty)
                {
                    throw(new Exception("Error, CropImage failed to allocate clipping rectangle"));
                }
                // Clone image using the crop rectangle.
                Bitmap croppedImage = loadedImage.Clone(rectangle,
                    System.Drawing.Imaging.PixelFormat.DontCare);
                // Make the cropped image the actual image that is shown.
                loadedImage = new Bitmap(croppedImage, 
                    rectangle.Size);
                if(loadedImage == null)
                {
                    throw(new Exception("Error, Image memory allocation failed"));
                }
            }
        }
        /// <summary>
        /// Flips and/or rotates the Image
        /// </summary>
        /// <requirement>F:image_flip</requirement>
        /// <requirement>F:image_rotate</requirement>
        /// <param name="rotateFlip">Enum type specifying the user requested rotation or flip operation</param>
        public void RotateFlipImage(RotateFlipType rotateFlip)
        {
            loadedImage.RotateFlip(rotateFlip);
            
        }
        /// <summary>
        /// Saves the current image
        /// used as test hook.
        /// </summary>
        public void SaveImage()
        {
            saveFileDialog.Filter = " jpg files (*.jpg)|*.jpg|gif files (*.gif)|*.gif| bmp files (*.bmp)|*.bmp|All files (*.*)|*.*";
            saveFileDialog.ShowDialog();
            loadedImage.Save(saveFileDialog.FileName);
            saveFileDialog.Dispose();
        }

		/// <summary>
		/// Accessor to get the currently loaded image.
		/// </summary>
		public Bitmap LoadedImage
		{
			get
			{
				return loadedImage;
			}
			set
			{
				backUpImage = (Bitmap) loadedImage.Clone();
				loadedImage = value;
			}
		}

        /// <summary>
        /// Test hook to get the List.
        /// </summary>
        public ArrayList GraphicsComponentList
        {
            get
            {
                return compList;
            }
        }

        /// <summary>
        /// Opens the file with the provided Name.
        /// </summary>
        /// <param name="fileName">name of the file to be opened</param>
        /// <requirement>F:image_load_and_save</requirement>
        public void LoadImage(string fileName)
        {	
            
            try
            {
                loadedImage = new Bitmap(fileName);
            }
            catch(Exception e)
            {
                Exception b = e;
                throw(new Exception("Check if the requested Image is in the path!"));
            }
            
            if(loadedImage == null)
            {
                throw(new Exception("Error, LoadImage with filename failed"));
            }
            // Back up image is set to current image at load time.
            backUpImage = (Bitmap)loadedImage.Clone();
            

        }
        /// <summary>
        /// Saves the current image
        /// with the provided fileName.
        /// </summary>
        /// <param name="fileName">Name under which the image is saved</param>
        public void SaveImage(string fileName)
        {
            loadedImage.Save(fileName);
        }

        /// <summary>
        /// The Draw method walks through the list
        /// of created components and calls the Draw
        /// method for each of the components.
        /// </summary>
        public override void Draw()
        {
            try
            {
                foreach(GraphicsComponent obj in compList)
                {
                    GraphicsComponent theComp = (GraphicsComponent)obj;   
                    theComp.Draw();     
                }
            }
            catch(Exception e)
            {
                throw(new Exception("In Picure.Draw", e));
            }
        }
    
        /// <summary>
        /// Method adds a new component to the list of 
        /// components.
        /// </summary>
        public override void Add(GraphicsComponent comp)
        {
            compList.Add(comp);
        }

        /// <summary>
        /// Removes a component form the list of components.
        /// </summary>
        public override void Remove(GraphicsComponent comp)
        {
            compList.Remove(comp);
        }

        /// <summary>
        /// Clears the list of components.
        /// </summary>
        public void ClearComponentList()
        {
            compList.Clear();
        }
        /// <summary>
        /// Goes through the list of components
        /// and checks if component is picked. If component
        /// returns true the component is returned to the
        /// caller.
        /// </summary>
        public Object PositionIsComponent(int x, int y)
        {
            foreach(GraphicsComponent obj in compList)
            {
                GraphicsComponent theComp = (GraphicsComponent)obj;
                if(theComp.IsPicked(x, y))
                {
                    return theComp;
                }
            }
            return null;
        }

        /// <summary>
        /// Draws the components in the list 
        /// into the Graphics object of the loaded
        /// picture to make changes permanent.
        /// </summary>
        public void ApplyPictureComponents()
        {
            // save current graphics object.
            Graphics tempDC = DeviceContext;
            // set Graohics object to the loaded image.
            DeviceContext = Graphics.FromImage(loadedImage);
            // Draw all components.
            Draw();
            // Reset the Graphics object to its
            // original value
            DeviceContext = tempDC;
            // Clear the list of components.
            ClearComponentList();

        }

        /// <summary>
        /// Restores the original loaded image and
        /// clears the list of components.
        /// </summary>
        public void Reset()
        {
            loadedImage = (Bitmap)backUpImage.Clone();
            compList.Clear();

        }

        private ArrayList compList;
        private Bitmap backUpImage;
       
	}
}
