using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Photo_Editor_Application
{
    /// <summary>
    /// Responsible for drawing a rectangular
    /// frame with the user specified color and
    /// border width. The border is using a PathGradientBrush
    /// where the center value is set to transparent.
    /// </summary>
    /// <requirement>F:image_graphics_special_effects</requirement>
    public class RectFrameComponent : GraphicsComponent
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        public RectFrameComponent()
        {
        }
        // Properties needed to draw the border.
        private Region region;
        private Rectangle outerRectangle;
        private Rectangle innerRectangle = new Rectangle();

        /// <summary>
        /// Draws a frame rectangle in a user specified width and color.
        /// </summary>
        public override void Draw()
        {
           
            // Calculate width and height of the rectangle.
            int width =  CalcViewportCoords(EndPoint).X - CalcViewportCoords(StartPoint).X;
            int height = CalcViewportCoords(EndPoint).Y - CalcViewportCoords(StartPoint).Y;
            // Check if width and height are zero. If so set them to one.
            if (width == 0) width = 1;
            if (height == 0) height = 1;

            // Calculate the otlining rectangle
            if(width < 0 && height < 0)
            {
                outerRectangle = new Rectangle(CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    Math.Abs(width), Math.Abs(height));
            }
            else if (width < 0)
            {
                outerRectangle = new Rectangle(CalcViewportCoords(EndPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    Math.Abs(width), height);
            }
            else if (height < 0)
            {
                outerRectangle = new Rectangle(CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(EndPoint).Y, 
                    width, Math.Abs(height));
            }
            else
            {
                outerRectangle = new Rectangle(CalcViewportCoords(StartPoint).X, 
                    CalcViewportCoords(StartPoint).Y, 
                    width, height);
            }

            // Calculate the border width and height.
            // If the Line Thickness is set to the maximum value 
            // of 20 then the whole rectangle is assumed to be border.
            int borderWidth = (int)(Math.Abs(width)*DrawingPen.Width)/40 ;
            int borderHeight = (int)(Math.Abs(height)*DrawingPen.Width)/40 ;
            innerRectangle.X = outerRectangle.X + borderWidth;
            innerRectangle.Y = outerRectangle.Y + borderHeight;
            innerRectangle.Width = outerRectangle.Width - borderWidth*2;
            innerRectangle.Height = outerRectangle.Height - borderHeight*2;
            // Create a graphics path object
            GraphicsPath gp = new GraphicsPath();
            // Add the enclosing Rectangle to the 
            // GraphicsPath.
            gp.AddRectangle(outerRectangle);
            // Use a PathGradientBrush to fill the region.
            PathGradientBrush pgb = new PathGradientBrush(gp);
            // Set the center color to Transparent = Alpha equals 0.
            pgb.CenterColor = Color.Transparent;
            // More than one surround color can be defined.
            // In this example the coloris set to the
            // value the user selected or black as default. 
            pgb.SurroundColors = new Color[]
                {
                    DrawingPen.Color
                };
            // Create a region the size of the ouline rectangle
            region = new Region(outerRectangle);
            // subtract the inner rectangle by doing Xor.
            // The result is a border with the specified size.
            region.Xor(innerRectangle);
            
            // Fill the region using the PathGradientBrush
            DeviceContext.FillRegion(pgb, region);
            // Dispose the brus, graphics path and the region
            region.Dispose();
            gp.Dispose();
            pgb.Dispose();
        }  
    }
}
